import boto3
import json
import uuid

from api import Response
from dynamodb import Table


def handler(event, context):
    """Runs a image"""

    method = event["httpMethod"]
    resource = event["resource"]

    key = (event.get("pathParameters") or {}).get("id", None)
    body = event.get("body", {})

    functions = {
        "GET": {
            "/execution/{id}": detail,
            "/execution": list,
        },
        "POST": {
            "/execution": create,
        },
        "DELETE": {
            "/execution/{id}": delete,
        },
        "PATCH": {
            "/execution/{id}": update,
        },
    }

    handler = functions.get(method, {}).get(resource)

    return handler(key=key, body=body)


def detail(key, *args, **kwargs):
    """Describes a entry"""

    table = Table("REGISTRY_TABLE_NAME")

    response = Response(200, table.get(key))

    return response.parse()


def list(*args, **kwargs):
    """Lists all entries"""
    pass


def delete(key, *args, **kwargs):
    """Deletes a entry"""
    pass


def update(key, body, *args, **kwargs):
    """Updates (patches) a entry"""
    pass


def create(body, *args, **kwargs):
    """Creates a entry"""

    table = Table("JOBS_TABLE_NAME")
    registry = Table("REGISTRY_TABLE_NAME")
    client = boto3.client("ecs")

    # Parses the body
    body = json.loads(body)

    # Generates a uuid and updates the table
    id = uuid.uuid4().hex
    table.update(id, {"status": "RECEIVED"})

    # Gets the task definition from the registry
    image = registry.get(body["image"])

    # Image not built yet
    if image is None:

        return Response(404, "Image does not exists.").parse()

    elif image["status"] != "SUCCEEDED":

        return Response(409, "Image is not built yet.").parse()

    # Default parameters
    parameters = {
        "cpu": body.get("cpu", image["definition"]["cpu"]),
        "memory": body.get("memory", image["definition"]["memory"]),
        "count": body.get("count", 1),
        "role": body.get("role", image["definition"]["roles"]["task"]),
        "environment": body.get("environment", []),
    }

    response = client.run_task(
        cluster="harbour",
        count=parameters["count"],
        launchType="FARGATE",
        group=id,
        networkConfiguration={
            "awsvpcConfiguration": {
                "subnets": [
                    "subnet-00f960fed27512fc7",
                    "subnet-029e4362fc65caf73",
                    "subnet-01b6fb88abc34f050",
                ],
                "securityGroups": [
                    "sg-0fec3abb27c2d3dec",
                ],
                "assignPublicIp": "DISABLED",
            }
        },
        overrides={
            "containerOverrides": [
                {
                    "name": "task",
                    "environment": [
                        {
                            "name": name,
                            "value": value,
                        }
                        for name, value in parameters["environment"].items()
                    ],
                    "cpu": int(parameters["cpu"]),
                    "memory": int(parameters["memory"]),
                    "memoryReservation": int(parameters["memory"]) - 50,
                },
            ],
            "cpu": parameters["cpu"],
            "memory": parameters["memory"],
            "taskRoleArn": parameters["role"],
        },
        startedBy="harbour:api",
        tags=[
            {"key": "database", "value": parameters["environment"]["DATABASE"]},
            {"key": "table", "value": parameters["environment"]["TABLE"]},
            {"key": "Name", "value":  "Dump." + parameters["environment"]["DATABASE"] + "." + parameters["environment"]["TABLE"]},
        ],
        taskDefinition=image["definition"]["arn"],
    )

    event = {
        "tasks": [
            {
                "id": None,
                "az": task["availabilityZone"],
                "placement": {
                    "availability_zone": task["availabilityZone"],
                },
                "attachments": task["attachments"],
                "cluster": {
                    "arn": task["clusterArn"],
                },
                "containers": [
                    {
                        "name": container["name"],
                        "arn": container["containerArn"],
                        "task": {
                            "arn": container["taskArn"],
                        },
                        "image": container["image"],
                        "cpu": int(container["cpu"]),
                        "memory_reservation": int(container["memoryReservation"]),
                    }
                    for container in task["containers"]
                ],
                "created_at": task["createdAt"],
                "group": task["group"],
                "launch_type": task["launchType"],
                "memory": int(task["memory"]),
                "cpu": int(task["cpu"]),
                "overrides": task["overrides"],
                "platform_version": task["platformVersion"],
                "started_by": task["startedBy"],
                "tags": task["tags"],
                "arn": task["taskArn"],
                "definition": task["taskDefinitionArn"],
            }
            for task in response["tasks"]
        ]
    }

    event["failures"] = response["failures"]

    table.update(id, event)

    response = Response(200, event)

    return response.parse()



import base64
import hashlib
import json
import os

import click
import requests

from harbour.client.parser import ImageNameParser
from harbour.client.object import Object
from harbour.client.artifact import Artifact


class Client:

    def __init__(self):

        # Builds a base session object
        self.session = requests.Session()

    def configure(self, domain, key, *args, **kwargs):
        """Creates a credentials file under ~/.harbour/credentials.json"""

        in_lambda = os.environ.get('IN_LAMBDA')

        if in_lambda:

            self.__domain = domain
            self.__key = key

            credentials = {
                'domain': domain,
                'key': key,
            }

            return credentials

        filedir = os.path.join(os.path.expanduser("~"), '.harbour')

        os.makedirs(filedir, exist_ok=True)

        filepath = os.path.join(filedir, 'credentials.json')

        with open(filepath, 'w+') as file:

            credentials = {
                'domain': domain,
                'key': key,
            }

            file.write(json.dumps(credentials, indent=3))

        credentials['filepath'] = filepath

        self.output(credentials)

        return credentials

    def authenticate(self):
        """Reads and stores the credentials in the object"""

        in_lambda = os.environ.get('IN_LAMBDA')

        if in_lambda:

            self.session.headers.update(
                {
                    'x-api-key': self.__key
                }
            )

            self.domain = self.__domain

            return self

        # Reads the credentials
        filepath = os.path.join(os.path.expanduser(
            "~"), '.harbour/credentials.json')

        with open(filepath, 'r') as file:

            self.__credentials = json.loads(file.read())

        # Injects it on the object
        self.session.headers.update(
            {
                'x-api-key': self.__credentials['key']
            }
        )

        self.domain = self.__credentials['domain']

        return self

    def request(self, request):
        """Requests a given endpoint"""

        self.authenticate()

        endpoint = request.pop('endpoint')
        endpoint = endpoint[1:] if endpoint.startswith('/') else endpoint

        request['url'] = f'https://{self.domain}/{endpoint}'

        response = self.session.request(**request)

        if response.status_code not in range(200, 399):

            raise Exception(
                f'[{request["url"]}]: ({response.status_code}) {response.reason}')

        else:
            return response.json()

    def output(self, message, *args, **kwargs):
        """Returns a formatted string output"""

        click.echo(json.dumps(message, indent=3))

    def object(self, name, *args, **kwargs):
        """Wrapper method to return objects"""

        return Object(self, name, *args, **kwargs)

    def register(self, path, name, *args, **kwargs):
        """Register a image against the repository"""

        # Parses the image name
        parser = ImageNameParser()
        image = parser.parse(name)

        # Checks if the image already exists. In this case, ask the user if
        # he wants to override it
        key = hashlib.md5(image.encode()).digest().hex()

        object = self.object('register').get(key)

        if object != {}:
            click.confirm('Image already exists. Overwrite?', abort=True)

        # Packages the code
        artifact = Artifact()

        filepath = artifact.package(path)

        # Requests the api to register the package
        with open(filepath, 'rb') as file:

            payload = base64.b64encode(file.read())

        object = self.object('register').create(key=image, object=payload)

        artifact.delete()

        entry = {
            'id': key,
            'image': image,
            'path': path,
            'name': name,
        }

        self.output(entry)

    def execute(self, image, *args, **kwargs):
        """Executes a image"""

        # Creates the image hash
        key = hashlib.md5(image.encode()).digest().hex()

        # Checks if the image exists
        object = self.object('register').get(key)

        if object is None:
            raise ValueError(f'Image {image} does not exists.')

        payload = {
            'image': key,
            **{k: v for k, v in kwargs.items() if v not in [None, {}, []]},
        }

        object = self.object('execution').create(payload)

        self.output(object)



import gzip
import io
import os
import uuid

import boto3


class Bucket:

    def __init__(self, bucket):

        self.name = os.getenv(bucket) or bucket
        self.client = boto3.client('s3', region_name='us-east-1')

    def scan(self, prefix='', *args, **kwargs):
        """Returns all the keys in a given prefix"""

        keys = []
        token = None

        while True:

            if token:

                response = self.client.list_objects_v2(
                    Bucket=self.name,
                    Prefix=prefix,
                    ContinuationToken=token,
                    MaxKeys=1000
                )

            else:

                response = self.client.list_objects_v2(
                    Bucket=self.name,
                    Prefix=prefix,
                    MaxKeys=1000
                )

            keys.extend(
                [c['Key'] for c in response.get('Contents', [])]
            )

            # Gets the next token iteration or breaks the loop
            token = response.get('NextContinuationToken', None)

            if token is None:
                break

        return keys

    def upload(self, filename, key, compress=False, delete=False, *args, **kwargs):
        """Uploads a file to the storage"""

        if not os.path.exists(filename):

            msg = 'The file {} could not be found'.format(filename)

            raise FileNotFoundError(msg)

        client = boto3.client('s3')

        if compress:

            # Defines a unique ID for the upload
            tmp_path = '/tmp/{}.gz'.format(uuid.uuid4().hex)

            # Writes the file as gzip
            with gzip.open(tmp_path, 'wt+') as gz:

                with open(filename, 'r') as file:

                    gz.write(file.read())

            # Sanitizes the key
            key += '.gz' if not key.endswith('.gz') else ''

            # Uploads the file
            response = client.upload_file(
                Filename=tmp_path,
                Bucket=self.name,
                Key=key
            )

            # Removes the temp file from disk
            os.remove(tmp_path)

        else:

            # Uploads the file
            response = client.upload_file(
                Filename=filename,
                Bucket=self.name,
                Key=key
            )

        # Removes the source file from disk
        if delete:

            os.remove(filename)

        return 's3://{}/{}'.format(self.name, key)

    def download(self, key, filename, overwrite=False, *args, **kwargs):
        """Downloads a key from the bucket and save as filename"""

        # Deletes the file before downloading
        if overwrite:
            try:
                os.remove(filename)
            except FileNotFoundError:
                pass

        response = self.client.download_file(
            Bucket=self.name,
            Key=key,
            Filename=filename
        )

        return filename

    def delete(self, keys, *args, **kwargs):
        """Deletes all the keys specified"""

        batch = []
        for key in keys:

            batch.append(key)

            # Hard limit of 1000 keys per call. Do not change it.
            if len(batch) == 1000:

                response = self.client.delete_objects(
                    Bucket=self.name,
                    Delete={
                        'Objects': [
                            {
                                'Key': k,
                            } for k in batch
                        ],
                    }
                )

                batch = []

        # Deletes the residual records
        else:
            response = self.client.delete_objects(
                Bucket=self.name,
                Delete={
                    'Objects': [
                        {
                            'Key': k,
                        } for k in batch
                    ],
                }
            )



import json
import os

import boto3

from dynamodb import Table

def handler(event, context):

    FIRELENS_IMAGE_MAP = {
        'us-east-1': '906394416424.dkr.ecr.us-east-1.amazonaws.com/aws-for-fluent-bit:latest',
        'us-east-2': '906394416424.dkr.ecr.us-east-2.amazonaws.com/aws-for-fluent-bit:latest',
        'us-west-1': '906394416424.dkr.ecr.us-west-1.amazonaws.com/aws-for-fluent-bit:latest',
        'us-west-2': '906394416424.dkr.ecr.us-west-2.amazonaws.com/aws-for-fluent-bit:latest',
        'ap-east-1': '449074385750.dkr.ecr.ap-east-1.amazonaws.com/aws-for-fluent-bit:latest',
        'ap-south-1': '906394416424.dkr.ecr.ap-south-1.amazonaws.com/aws-for-fluent-bit:latest',
        'ap-northeast-2': '906394416424.dkr.ecr.ap-northeast-2.amazonaws.com/aws-for-fluent-bit:latest',
        'ap-southeast-1': '906394416424.dkr.ecr.ap-southeast-1.amazonaws.com/aws-for-fluent-bit:latest',
        'ap-southeast-2': '906394416424.dkr.ecr.ap-southeast-2.amazonaws.com/aws-for-fluent-bit:latest',
        'ap-northeast-1': '906394416424.dkr.ecr.ap-northeast-1.amazonaws.com/aws-for-fluent-bit:latest',
        'ca-central-1': '906394416424.dkr.ecr.ca-central-1.amazonaws.com/aws-for-fluent-bit:latest',
        'eu-central-1': '906394416424.dkr.ecr.eu-central-1.amazonaws.com/aws-for-fluent-bit:latest',
        'eu-west-1': '906394416424.dkr.ecr.eu-west-1.amazonaws.com/aws-for-fluent-bit:latest',
        'eu-west-2': '906394416424.dkr.ecr.eu-west-2.amazonaws.com/aws-for-fluent-bit:latest',
        'eu-west-3': '906394416424.dkr.ecr.eu-west-3.amazonaws.com/aws-for-fluent-bit:latest',
        'eu-north-1': '906394416424.dkr.ecr.eu-north-1.amazonaws.com/aws-for-fluent-bit:latest',
        'me-south-1': '741863432321.dkr.ecr.me-south-1.amazonaws.com/aws-for-fluent-bit:latest',
        'sa-east-1': '906394416424.dkr.ecr.sa-east-1.amazonaws.com/aws-for-fluent-bit:latest',
        'gov-east-1': '161423150738.dkr.ecr.us-gov-east-1.amazonaws.com/aws-for-fluent-bit:latest',
        'gov-west-1': '161423150738.dkr.ecr.us-gov-west-1.amazonaws.com/aws-for-fluent-bit:latest',
        'cn-north-1': '128054284489.dkr.ecr.cn-north-1.amazonaws.com.cn/aws-for-fluent-bit:latest',
        'cn-northwest-1': '128054284489.dkr.ecr.cn-northwest-1.amazonaws.com.cn/aws-for-fluent-bit:latest',
    }

    client = boto3.client('ecs')

    table = Table('REGISTRY_TABLE_NAME')

    # {
    #     "version": "0",
    #     "id": "13cde686-328b-6117-af20-0e5566167482",
    #     "detail-type": "ECR Image Action",
    #     "source": "aws.ecr",
    #     "account": "123456789012",
    #     "time": "2019-11-16T01:54:34Z",
    #     "region": "us-west-2",
    #     "resources": [],
    #     "detail": {
    #         "result": "SUCCESS",
    #         "repository-name": "my-repo",
    #         "image-digest": "sha256:7f5b2640fe6fb4f46592dfd3410c4a79dac4f89e4782432e0378abcd1234",
    #         "action-type": "PUSH",
    #         "image-tag": "latest"
    #     }
    # }

    for record in event['Records']:

        event = json.loads(record['body'])

        response = client.register_task_definition(
            family=event['detail']['image-tag'],
            taskRoleArn='arn:aws:iam::417984092495:role/ECSTaskRole',
            executionRoleArn='arn:aws:iam::417984092495:role/ECSExecutionRole',
            networkMode='awsvpc',
            containerDefinitions=[
                {
                    'name': 'task',
                    'image': f'{event["account"]}.dkr.ecr.{event["region"]}.amazonaws.com/{event["detail"]["repository-name"]}:{event["detail"]["image-tag"]}',
                    'cpu': 2048,
                    'memory': 4096,
                    'memoryReservation': 4046,
                    'essential': True,
                    'dependsOn': [
                        {
                            'containerName': 'logrouter',
                            'condition': 'START'
                        },
                    ],
                    'startTimeout': 30,
                    'stopTimeout': 30,
                    'privileged': False,
                    'logConfiguration': {
                        'logDriver': 'awsfirelens',
                        'options': {
                            'Name': 'firehose',
                            'region': event['region'],
                            'delivery_stream': os.getenv('LOG_STREAM_NAME'),
                        },
                    },
                },
                {
                    'name': 'logrouter',
                    'image': FIRELENS_IMAGE_MAP.get(event['region']),
                    'firelensConfiguration': {
                        'type': 'fluentbit',
                        'options': {
                            'config-file-type': 'file',
                            'config-file-value': '/fluent-bit/configs/parse-json.conf',
                        },
                    },
                    'essential': True,
                    'memoryReservation': 50,
                    'logConfiguration': {
                        'logDriver': 'awslogs',
                        'options': {
                            'awslogs-group': '/harbour/ecs/firelens',
                            'awslogs-region': os.getenv('AWS_REGION'),
                            'awslogs-create-group': 'true',
                            'awslogs-stream-prefix': 'task'
                        },
                    },
                },
            ],
            requiresCompatibilities=['EC2', 'FARGATE'],
            cpu="2048",
            memory="4096"
        )

        table.update(
            event['detail']['image-tag'],
            {
                'definition': {
                    'arn': response['taskDefinition']['taskDefinitionArn'],
                    'family': response['taskDefinition']['family'],
                    'revision': response['taskDefinition']['revision'],
                    'roles': {
                        'task': response['taskDefinition']['taskRoleArn'],
                        'execution': response['taskDefinition']['executionRoleArn'],
                    },
                    'requirements': [r['name'] for r in response['taskDefinition']['requiresAttributes']],
                    'compatibilities': response['taskDefinition']['compatibilities'],
                    'cpu': response['taskDefinition']['cpu'],
                    'memory': response['taskDefinition']['memory'],
                },
            },
        )



import hashlib

class Object:

    def __init__(self, client, name, *args, **kwargs):

        self.client = client
        self.name = name

    def get(self, key, hash=False):
        """Returns a object"""

        if hash:
            key = hashlib.md5(key.encode()).digest().hex()

        request = {
            'method': 'GET',
            'endpoint': f'/{self.name}/{key}',
        }

        return self.client.request(request)

    def create(self, object, key=None, hash=False):
        """Creates a object"""

        request = {
            'method': 'POST',
        }

        if hash:
            key = hashlib.md5(key.encode()).digest().hex()

        # Switches the endpoint based on the key
        if key is None:

            request['endpoint'] = f'/{self.name}'

        else:

            request['endpoint'] = f'/{self.name}/{key}'

        # Compress and send
        if isinstance(object, (bytes, bytearray)):

            request['data'] = object

        elif isinstance(object, dict):

            request['json'] = object

        return self.client.request(request)



import json

from decimal import Decimal
from datetime import datetime, date

class Response:

    def __init__(self, code, body, *args, **kwargs):

        self.code = code
        self.body = body

    def format(self, obj, *args, **kwargs):
        """Default json formatter"""

        if isinstance(obj, datetime):

            return obj.strftime('%Y-%m-%d %H:%M:%S.%f')

        elif isinstance(obj, date):

            return obj.strftime('%Y-%m-%d')

        elif isinstance(obj, Decimal):

            return format(obj, '.8')

        elif isinstance(obj, bytes):

            return obj.decode(errors='ignore')

        elif isinstance(obj, timedelta):

            return str(obj)

        elif isinstance(obj, set):

            return ','.join(list(obj))

        else:

            raise TypeError ("Type %s not serializable" % type(obj))

    def parse(self, *args, **kwargs):
        """Returns a parsed response"""

        if isinstance(self.body, (dict, list)):

            self.body = json.dumps(self.body, default=self.format)

        elif isinstance(self.body, (str)):
            pass

        elif self.body is None:
            self.body = json.dumps({})

        else:
            self.code = 500
            self.body = f'Invalid response format {type(self.body)}'

        response = {
            'statusCode': self.code,
            'body': self.body,
        }

        return response



import os

class Parser:

    def __init__(self):
        pass

class ImageNameParser(Parser):

    def __init__(self):
        pass

    def parse(self, name, *args, **kwargs):
        """Sanitizes the image name. Adding :latest to non tagged images"""

        # Splits the image name
        name = name.split(':')

        # The image name is always the first
        image = name.pop(0)

        if len(name) >= 1:
            tag = ''.join(name).replace(':', '')

        else:
            tag = 'latest'

        return f'{image}:{tag}'



class Table:

    def __init__(self, db, name, *args, **kwargs):

        self.schema = db.credentials.get('db', db.credentials.get('dbname'))
        self.name = name

        # Default state
        self.sql = None
        self.columns = '*'
        self.filter = None
        self._limit = None
        self.order = None
        self.primary_key = None
        self._sequence = None

        # Database connections
        self.db = db
        self.conn, self.curs = db.conn, db.curs
        self.host = db.credentials['host']

    def select(self, columns, *args, **kwargs):
        """Defines the columns selected"""

        # Type checking
        if columns == '*':

            self.columns = ['*']

        elif isinstance(columns, str):

            self.columns = [columns]

        elif not isinstance(columns, list):

            raise TypeError('columns must be str or list')

        return self

    def order_by(self, order, *args, **kwargs):
        """Appends a ORDER BY clausule to the query"""

        if not isinstance(order, dict):
            raise TypeError('order must be a dict {"column": "ASC|DESC"}')

        for value in order.values():

            if value not in ['ASC', 'DESC']:
                raise AttributeError('order directoin must be ASC or DESC')

        self.order = order

        return self

    def sequence(self, value: str):
        self._sequence = value
        
        return self

    def where(self, filter:str):
        """Appends a where clausule to the query.
        Premisses:
            This filter is only a string 
        """

        if filter is not None:
            if "where " in filter.lower():
                filter = filter.replace("where ", "")
            if filter[:3].lower() == "and":
                filter = filter[3:]
        self.filter = filter

        return self

    def limit(self, limit, *args, **kwargs):
        """Appends a LIMIT clausule to the query"""

        if not isinstance(limit, int):
            raise TypeError('limit must be a int')

        if limit != 0:

            self._limit = limit

        return self

    def build(self, *args, **kwargs):
        """Builds the SQL"""

        self.sql = "/*Application: Tethys - Dumper*/\n"
        self.sql += "SELECT {}".format(', '.join(self.columns))
        
        if self._sequence is None:
            if self.db.engine == 'postgresql':
                self.sql += ", CAST(extract(epoch from now() at time zone 'utc' at time zone 'utc') * POW(10, 6) AS DECIMAL(25,9)) AS _sequence\n"
                self.sql += 'FROM "{}".{}\n'.format(self.schema, self.name)
            else:
                self.sql += ", CAST(UNIX_TIMESTAMP() * POW(10, 6) AS DECIMAL(25,9)) AS _sequence\n"
                self.sql += "FROM `{}`.{}\n".format(self.schema, self.name)
        else:
            if self.db.engine == 'postgresql':
                self.sql += f", {self._sequence} AS _sequence\n"
                self.sql += 'FROM "{}".{}\n'.format(self.schema, self.name)
            else:
                self.sql += f", {self._sequence} AS _sequence\n"
                self.sql += "FROM `{}`.{}\n".format(self.schema, self.name)


        if self.filter is not None:

            self.sql += "WHERE\n\t1=1 AND {}\n".format(self.filter)

        if self.order is not None:

            self.sql += "ORDER BY {}\n".format(','.join(
                ['{} {}'.format(k, v) for k, v in self.order.items()]))

        if self._limit is not None:
            self.sql += "LIMIT {}".format(self._limit)

        return self

    def execute(self):
        """Executes a query to check the table metadata and then executes the
        built sql"""

        self.build()

        self.curs.execute(self.sql)

        return self

    @property
    def fields(self):
        """Returns the table schema"""

        from harbour.database.mappings import MYSQL_TYPE_MAP, POSTGRESQL_TYPE_MAP

        return {
            field[0]: {
                'type':
                POSTGRESQL_TYPE_MAP.get(field[1]) if self.db.engine
                == 'postgresql' else MYSQL_TYPE_MAP.get(field[1])
            }
            for field in self.curs.description if field[0] != '_sequence'
        }

    @property
    def pk(self):
        """Returns the table primary key"""

        # This a cache. Do not remove!
        if self.primary_key is None:

            sql = f"""
            SELECT
                u.COLUMN_NAME AS column_name
            FROM information_schema.TABLE_CONSTRAINTS c
            INNER JOIN information_schema.KEY_COLUMN_USAGE u
                ON c.TABLE_SCHEMA = u.TABLE_SCHEMA
                AND c.TABLE_NAME = u.TABLE_NAME
                AND c.CONSTRAINT_NAME = u.CONSTRAINT_NAME
            WHERE
                c.TABLE_SCHEMA = '{self.schema}'
                AND c.TABLE_NAME = '{self.name}'
                AND c.CONSTRAINT_TYPE = 'PRIMARY KEY'
            ORDER BY u.ORDINAL_POSITION ASC;
            """

            self.curs.execute(sql)

            self.primary_key = [
                row['column_name'] for row in self.curs.fetchall()
            ]

        return self.primary_key

    def fetchall(self):
        """Fetches all the records."""

        for row in self.curs.fetchall():
            yield row

    def fetchmany(self, n=1000):
        """Fetches as many records as specified. Defaul: 1000"""

        return self.curs.fetchmany(n)

    def __iter__(self):
        """Iterates over the table records."""

        return self.fetchall()



import setuptools

setuptools.setup(
    name="harbour",
    version="0.0.20",
    author="ebanx-dataops",
    author_email="dataops@ebanx.com",
    description="This project aims to make it easier to create images in AWS ECR. There are some implementations already in place like Dump tables from Mysql and Postgres databases.",
    url="https://github.com/ebanx/harbour",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    entry_points={"console_scripts": ["harbour = harbour.cli:cli"]},
    install_requires=[
        "click==7.0",
        "requests==2.22.0",
        "boto3==1.11.9",
        "structlog==20.1.0",
        "PyMySQL==0.9.3",
        "psycopg2-binary==2.8.6",
    ],
    extras_require={
        "test": [],
        "build": [
            "twine",
        ],
    },
)



import json
import os

from dynamodb import Table

def handler(event, context):

    table = Table('REGISTRY_TABLE_NAME')

    for record in event['Records']:

        body = json.loads(record['body'])

        # Extracts the key from the event
        key = [
            env['value'] for env in body['detail']['additional-information']['environment']['environment-variables']
            if env['name'] == 'IMAGE_TAG'
        ][0]


        # Any phase change
        if body['detail-type'] == 'CodeBuild Build State Change':

            event = {
                'status': body['detail']['build-status'],
                'build': {
                    'id': body['detail']['build-id'],
                    'project': {
                        'name': body['detail']['project-name'],
                    },
                    'source': body['detail']['additional-information']['source'],
                    'logs': body['detail']['additional-information']['logs'],
                }
            }

            table.update(key, event)

        # Any status change
        elif body['detail-type'] == 'CodeBuild Build Phase Change':

            event = {
                'status': body['detail']['completed-phase']
            }

            table.update(key, event, mode='PATCH')
